'****************************************************************
'*  Name    : RSOutDigitsKeypad_25K20.inc                       *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2018 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 23/07/2018                                        *
'*  Version : 1.0                                               *
'*  Notes   : Proton Basic Compiler V3.6.1.7                    *
'*          : Proteus Version 8.2 SP0                           *
'*          : Original Code written by Les Johnson              *
'* Purpose  : Library for Multiplexing 2 to 8 digits LED display*
'* Hardware : with multiplexer to reduce pins count             *
'*          : 74HC238 + UDN2981A or ULN2803                     *
'* Software : Using the modified RsOut command                  *
'****************************************************************

(*
Recommended identifiers for the RsOut command.
The modifiers are listed below:
Modifier Operation
    Bin{1..To number of digits} Send binary digits
    Dec{0..To number of digits} Send decimal digits (amount of digits after decimal point with floating point)
    Hex{1..To number of digits} Send hexadecimal digits
    SBin{1..To number of digits} Send signed binary digits
    SDec{0..limited} Send signed decimal digits
    SHex{1..To number of digits} Send signed hexadecimal digits

Mode:
Examples:
    RsOut Dec MyWord,CR
    RsOut Dec3 MyFloat,CR
    RsOut HEX3 MyWord,CR
    RsOut Bin4 MyWord,CR 

Or to avoid tipo errata, use the Macros.
    PrintDispInt(Dec, MyDword)
    PrintDispInt(Bin8, MyWord)  
    PrintDispFlt(Dec3, MyFloat)    
    PrintDispSign(SDec, MySDword)
    PrintDispSign(Bin7, MySDword)      

It is possible to print some ASCII characters like this (use low case characters only).
    MyASCIIarray = "STOP"
    RsOut Str MyASCIIarray\4,CR

    MyASCIIarray = "INPUT"
    RsOut Str MyASCIIarray\5,CR

    MyASCIIarray = "OUPUT"
    RsOut Str MyASCIIarray\6,CR

Note: The ":",";","<","=",">","?","@" mean a SPACE in the display.  
*)
'/==============================================================================
$ifndef _MultiplexDigits
$define _MultiplexDigits
'/==============================================================================
    '/ Uncomment the next line to disable the Proteus test.
    $define _ProteusTest
'/------------------------------------------------------------------------------
'    Device = 18F23K20
'    Device = 18F24K20
    Device = 18F25K20
'    Device = 18F26K20
'    Device = 18F43K20
'    Device = 18F44K20    
'    Device = 18F45K20    
'    Device = 18F46K20    
       
    $ifdef _ProteusTest
    Declare Xtal = 8    ' For Proteus testing
    $else
    Declare Xtal = 64   ' For real PIC (64Mhz)
    $endif

    Declare Optimiser_Level = 3
    Declare Dead_Code_Remove = On
    Declare Float_Display_Type = Fast
    Declare Show_System_Variables = On
    Declare Create_Coff = On
    Declare Watchdog = OFF
    Declare Bootloader = OFF
    Declare Eeprom_Address $F00000  ' This address changes with K40/K42 PICs.

'/==============================================================================
'/ Configuration of the Config Fuses.
    ' DEFINE THE PLL: On / OFF
    $define PLL_ConfigFuses OFF

    '/ Enable the to use the internal Oscillator.
    '/ Uncomment the next line to enable the configuration.
'    $define _InternalOSC_

    '/ Uncomment the Line to use the CLOCK OUT FUNCTION: 
    '/ If CLKOUT function is enabled, CLKOUT on RA6 & Port function On RA7 otherwize Port function On RA6 & RA7.     
'    $Define _CLKOUT_Function_  

'/------------------------------------------------------------------------------
'/ Configuration of the displays hardware.
    '/ Declare Display Mode (0 => Common Cathode) (1 => Common Anode) with transistors
    $define DisplayMode 0   

    '/ Declare Number of Digits (3,4,5,6,7,8)
    $define NumberDigits 8

    '/ Declare the Delay to see the Error in the Display. (MS)
    $define ErrorDelayMS 2000

'/------------------------------------------------------------------------------
'/ Configuration of the Keypad hardware.
    '/ Declare the Keypad option.
    '/ Comment the next line to disable the Keypad option
    '/ Define the Keypad type. (4X4/4X3)
    $define _Keypad 4X4

    '/ Declare the Keypad TimeOut.
    $define KeypadTimeoutMS 2000

    '/ Declare number or digits for Password. (4 to 8)
    '/ Comment the next line to disable the Password option
    $define PassWordDigits 6

'/------------------------------------------------------------------------------
'/ Declare the Timer used for scanning of the display. (one only)
'/ Comment one line to disable the option.
'    $define ScanTimer0
    $define ScanTimer2

'/------------------------------------------------------------------------------
'/ Declare the delay for buzzer when a key is pressed. 
'/ This delay must to be adjusted with the Interrupt Timing.
'/ Comment the next line to disable this option.
    $define BuzzerDelayMS 250
    
'/==============================================================================
'/==============================================================================
    $if _device != _18F23K20 And _device != _18F24K20 And _device != _18F25K20 And _device != _18F26K20 And _device != _18F43K20 And _device != _18F44K20 And _device != _18F45K20 And _device != _18F46K20
    $error "This library is for the PIC18FxxK20 devices only. Please write a new library"
    $endif

    $ifdef _ProteusTest
        $if _xtal != 8
        $error "It is better to use the Declare Xtal = 8 for Proteus"
        $endif   
        $ifdef _InternalOSC_
        $error "The internal oscillator does not run with Proteus"
        $endif
        $if PLL_ConfigFuses = On
        $error "It is better to use [PLL_ConfigFuses = OFF] with Proteus"
        $endif        
    $endif  
'/------------------------------------------------------------------------------
'/ Check validity.
    $ifndef _Keypad
        $undef BuzzerDelayMS
        $undef _BuzzerOption
        $undef PassWordDigits
    $else
        '/ Check the Keypad
        $if _Keypad = 4X4
            $if NumberDigits < 4
            $error "The number of Digits must be >= 4 for 4X4 keypad"
            $endif
        $endif
        $if _Keypad = 4X3
            $if NumberDigits < 3
            $error "The number of Digits must be >= 3 for 4X3 keypad"
            $endif
        $endif
        '/ To see the values in the asm file.
        $ifdef BuzzerDelayMS
            $define _BuzzerOption
            #define __BuzzerOption 1 
        $endif

        #define __ErrorDelay ErrorDelayMS  

        $ifdef PassWordDigits
            $if PassWordDigits < 4 Or PassWordDigits > 8
            $error "NO such value for Password digits"
            $endif
            #define __Password_Option PassWordDigits
        $endif
    $endif

'/ Declare the interruption flags (automatic).
    $ifdef ScanTimer0
    $define DisPlayInterruptFlag INTCONBits_TMR0IF
    $define DisPlayInterruptEnable INTCONBits_TMR0IE
    #define __INTERRUPT_TIMER0 1    '/ To check the TIMER0  in the ASM file.
    $endif

    $ifdef ScanTimer2    
    $define DisPlayInterruptFlag PIR1Bits_TMR2IF
    $define DisPlayInterruptEnable PIE1Bits_TMR2IE
    #define __INTERRUPT_TIMER2 1    '/ To check the TIMER2 in the ASM file.
    $endif

'/ Declare Hardware type (8 displays with 1 PORT + demultiplexer)
    $if DisplayMode = 0
        $define _Hardware 0 '/ Common Cathode: 74HC238 + UDL2803  
        #define __HARDWARE1PORT_74HCT238_ULN2803 1   '/ To check the Hardware in the ASM file.
    $else
        $define _Hardware 1 '/ Common Anode: 74HC238 + UDN2981A
        #define __HARDWARE1PORT_74HCT238_UDN2981A 1  '/ To check the Hardware in the ASM file.
    $endif 
'===============================================================================
'/ Declare the Carriage Return
    $ifndef CR
    $define CR 13
    $endif
'/ Declare the Line Feed
    $ifndef LF
    $define LF 10
    $endif

'/ Calculate the better option for Table Data (automaticly).
    '/ Declare Table Data (0 from ROM, 1 from eeprom)
    $if _eeprom > 0
        $define TableData 1    
        #define __TableDataEeprom 1 '/ To check the Table Mode in the ASM file.
    $else
        $define TableData 0    
        #define __TableDataROM 1    '/ To check the Table Mode in the ASM file.
    $endif
                    
'/ Declare the outputs of scanning.
    $if DisplayMode = 0
        $define TurnOffDisp 0                   '/ Turn Off For transistor NPN    
        $define TurnOnDisp 1                    '/ Turn On For transistor NPN
        #define __DisplayMode_CommonCathode 1   '/ To check the Display Mode in the ASM file.
    $else
        $define TurnOffDisp 1                   '/ Turn Off For transistor PNP    
        $define TurnOnDisp 0                    '/ Turn On For transistor PNP  
        #define __DisplayMode_CommonAnode 1     '/ To check the Display Mode in the Asm file.
    $endif 

    $if NumberDigits < 3 Or NumberDigits > 8
    $error "NO such digits number allowed"
    $endif
    $if DisplayMode > 1
    $error "The DisplayMode must be 0 or 1"
    $endif
    $if TableData > 1
    $error "The Table of Data must be 0 = ROM or 1 = EEPROM"
    $endif

'/ Check a timer
    $if !_defined(ScanTimer0) And !_defined(ScanTimer2)
    $error "Timer0 or Timer2 must be declared"    
    $endif    

    '/--------------------------------------------------------------------------
'/ Configuration of the RsOut Serial comunication
    Declare RsOut_Pin PORTB.0       '/ It is not a real pin.
    Declare RsOut_Mode Inverted   
    Declare Serial_Baud 9600    
'    Declare Serial_Data 8
'    Declare RsOut_Pace 5
'    Declare Shift_DelayUs 2
'===============================================================================
    GoTo _MultiplexDigitsExit
'===============================================================================
'/**** Added by Fuse Configurator ****
'/ Use the Fuse Configurator plug-in to change these settings

$if _device = _18F23K20
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 8) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	    ;Fail-Safe Clock Monitor disabled
    IESO = OFF	     ;Oscillator Switchover mode disabled
    PWRT = On	    ;PWRT enabled
    BOREN = On	     ;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	      ;VBOR set to 2.2 V nominal
    WDTEN = OFF	    ;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 1024	    ;1:1024
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	    ;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	     ;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	    ;Stack full/underflow will cause Reset
    LVP = OFF	    ;Single-Supply ICSP disabled
    XINST = OFF	    ;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	    ;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	    ;Block 0 (000200-000FFFh) not code-protected
    CP1 = OFF	    ;Block 1 (001000-001FFFh) not code-protected
    CPB = OFF	    ;Boot block (000000-0001FFh) not code-protected    
    CPD = On	    ;Data EEPROM code-protected
    WRT0 = OFF	     ;Block 0 (000200-000FFFh) not write-protected 
    WRT1 = OFF	     ;Block 1 (001000-001FFFh) not write-protected
    WRTC = On	     ;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	      ;Boot Block (000000-0001FFh) not write-protected  
    WRTD = OFF	     ;Data EEPROM not write-protected
    EBTR0 = OFF	    ;Block 0 (000200-000FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	    ;Block 1 (001000-001FFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	    ;Boot Block (000000-0001FFh) not protected from table reads executed in other blocks
Config_End
$endif
'/------------------------------------------------------------------------------
$if _device = _18F24K20
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 8) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	    ;Fail-Safe Clock Monitor disabled
    IESO = OFF	     ;Oscillator Switchover mode disabled
    PWRT = On	      ;PWRT enabled
    BOREN = On	     ;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	      ;VBOR set to 2.2 V nominal
    WDTEN = OFF	        ;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 1024	    ;1:1024
    CCP2MX = PORTC	     ;CCP2 input/output is multiplexed with RC1
    PBADEN = On	        ;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	 ;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-001FFFh) not code-protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code-protected
    CPB = OFF	;Boot block (000000-0007FFh) not code-protected    
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-001FFFh) not write-protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	;Boot Block (000000-0007FFh) not write-protected 
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
$endif
'/------------------------------------------------------------------------------
$if _device = _18F25K20
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 8) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	;VBOR set to 2.2 V nominal
    WDTEN = OFF	;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 1024	    ;1:1024
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-001FFFh) not code-protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code-protected
    CP2 = OFF	;Block 2 (004000-005FFFh) not code-protected
    CP3 = OFF	;Block 3 (006000-007FFFh) not code-protected
    CPB = OFF	;Boot block (000000-0007FFh) not code-protected    
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-001FFFh) not write-protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write-protected
    WRT2 = OFF	;Block 2 (004000-005FFFh) not write-protected
    WRT3 = OFF	;Block 3 (006000-007FFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	;Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks   
Config_End
$endif
'/------------------------------------------------------------------------------
$if _device = _18F26K20
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 8) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	    ;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 18	    ;VBOR set to 1.8 V nominal
    WDTEN = OFF	;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 1024	    ;1:1024
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	    ;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	    ;Block 0 (000800-003FFFh) not code-protected
    CP1 = OFF	    ;Block 1 (004000-007FFFh) not code-protected
    CP2 = OFF	    ;Block 2 (008000-00BFFFh) not code-protected
    CP3 = OFF	    ;Block 3 (00C000-00FFFFh) not code-protected
    CPB = OFF	    ;Boot block (000000-0007FFh) not code-protected
    CPD = On	    ;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-003FFFh) not write-protected
    WRT1 = OFF	;Block 1 (004000-007FFFh) not write-protected
    WRT2 = OFF	;Block 2 (008000-00BFFFh) not write-protected
    WRT3 = OFF	;Block 3 (00C000h-00FFFFh) not write-protected
    WRTC = On	    ;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	    ;Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks    
Config_End
$endif
'/------------------------------------------------------------------------------
$if _device = _18F43K20
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 8) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	;VBOR set to 2.2 V nominal
    WDTEN = OFF	;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 1024	    ;1:1024
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000200-000FFFh) not code-protected
    CP1 = OFF	;Block 1 (001000-001FFFh) not code-protected
    CPB = OFF	;Boot block (000000-0001FFh) not code-protected  
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000200-000FFFh) not write-protected
    WRT1 = OFF	;Block 1 (001000-001FFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	;Boot Block (000000-0001FFh) not write-protected 
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000200-000FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (001000-001FFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0001FFh) not protected from table reads executed in other blocks  
Config_End
$endif
'/------------------------------------------------------------------------------
$if _device = _18F44K20
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 8) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	;VBOR set to 2.2 V nominal
    WDTEN = OFF	;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 1024	    ;1:1024
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-001FFFh) not code-protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code-protected
    CPB = OFF	;Boot block (000000-0007FFh) not code-protected
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-001FFFh) not write-protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	;Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
$endif
'/------------------------------------------------------------------------------
$if _device = _18F45K20
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 8) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	;VBOR set to 2.2 V nominal
    WDTEN = OFF	;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 1024	    ;1:1024
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-001FFFh) not code-protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code-protected
    CP2 = OFF	;Block 2 (004000-005FFFh) not code-protected
    CP3 = OFF	;Block 3 (006000-007FFFh) not code-protected
    CPB = OFF	;Boot block (000000-0007FFh) not code-protected
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-001FFFh) not write-protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write-protected
    WRT2 = OFF	;Block 2 (004000-005FFFh) not write-protected
    WRT3 = OFF	;Block 3 (006000-007FFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	;Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
$endif
'/------------------------------------------------------------------------------
$if _device = _18F46K20
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 8) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	;VBOR set to 2.2 V nominal
    WDTEN = OFF	    ;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 1024	    ;1:1024
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	    ;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-003FFFh) not code-protected
    CP1 = OFF	;Block 1 (004000-007FFFh) not code-protected
    CP2 = OFF	;Block 2 (008000-00BFFFh) not code-protected
    CP3 = OFF	;Block 3 (00C000-00FFFFh) not code-protected
    CPB = OFF	;Boot block (000000-0007FFh) not code-protected
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-003FFFh) not write-protected
    WRT1 = OFF	;Block 1 (004000-007FFFh) not write-protected
    WRT2 = OFF	;Block 2 (008000-00BFFFh) not write-protected
    WRT3 = OFF	;Block 3 (00C000h-00FFFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	;Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
$endif
'/**** End of Fuse Configurator Settings ****
'===============================================================================
    On_Hardware_Interrupt GoTo MULTILEX_DIGITS		' Point to the interrupt handler and ENABLE global interrupts
'===============================================================================       
'/ Name          : Declare variable of the library.
'/ Input         : None
'/ Return Value  : None
'/ Parameters    : Using system variables the routines could run faster.
'/ Notes         : Create a Dummy Array after the DISP_STRING to avoid crashing the PIC,
'/               : because too much bytes received.
'/ 
$if NumberDigits = 8
    #define __NumberDigits 8                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_STRING 9
    #define __LenghtDISP_STRING 9           '/ Assembler definition. For testing the variables.

    Dim DISP_STRING As String * LenghtDISP_STRING 
    Dim DispDummyArray[10] As Byte          '/ Create a Dummy Array after the DISP_STRING to avoid crashing the PIC
    Dim ONES As DISP_STRING#0
    Dim TENS As DISP_STRING#1
    Dim HUNDREDS As DISP_STRING#2
    Dim THOUSANDS As DISP_STRING#3
    Dim TENTHOUSANDS As DISP_STRING#4
    Dim HUNDREDTHOUSANDS As DISP_STRING#5
    Dim THOUSANDTHOUSANDS As DISP_STRING#6
    Dim TENTHDTHOUSANDS As DISP_STRING#7
    
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As Byte System '/ Variable for interrupt routine.
    Dim PRINT_TENTHDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
    #ifMacro- _mEnterValue
    Dim DISPVAL As Dword System
    #endIfMacro-
$endif

$if NumberDigits = 7
    #define __NumberDigits 7                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_STRING 8
    #define __LenghtDISP_STRING 8            '/ Assembler definition. For testing the variables.

    Dim DISP_STRING As String * LenghtDISP_STRING
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_STRING to avoid crashing the PIC
    Dim ONES As DISP_STRING#0
    Dim TENS As DISP_STRING#1
    Dim HUNDREDS As DISP_STRING#2
    Dim THOUSANDS As DISP_STRING#3
    Dim TENTHOUSANDS As DISP_STRING#4
    Dim HUNDREDTHOUSANDS As DISP_STRING#5
    Dim THOUSANDTHOUSANDS As DISP_STRING#6
    
    Dim PRINT_ONES As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System       '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    #ifMacro- _mEnterValue
    Dim DISPVAL As Dword System
    #endIfMacro-
$endif

$if NumberDigits = 6
    #define __NumberDigits 6                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_STRING 7
    #define __LenghtDISP_STRING 7            '/ Assembler definition. For testing the variables.

    Dim DISP_STRING As String * LenghtDISP_STRING 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_STRING to avoid crashing the PIC
    Dim ONES As DISP_STRING#0
    Dim TENS As DISP_STRING#1
    Dim HUNDREDS As DISP_STRING#2
    Dim THOUSANDS As DISP_STRING#3
    Dim TENTHOUSANDS As DISP_STRING#4
    Dim HUNDREDTHOUSANDS As DISP_STRING#5

    Dim PRINT_ONES As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System       '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
    #ifMacro- _mEnterValue
    Dim DISPVAL As Dword System
    #endIfMacro-
$endif

$if NumberDigits = 5
    #define __NumberDigits 5                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_STRING 6
    #define __LenghtDISP_STRING 6            '/ Assembler definition. For testing the variables.

    Dim DISP_STRING As String * LenghtDISP_STRING 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_STRING to avoid crashing the PIC
    Dim ONES As DISP_STRING#0
    Dim TENS As DISP_STRING#1
    Dim HUNDREDS As DISP_STRING#2
    Dim THOUSANDS As DISP_STRING#3
    Dim TENTHOUSANDS As DISP_STRING#4
    
    Dim PRINT_ONES As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System       '/ Variable for interrupt routine.
    #ifMacro- _mEnterValue
    Dim DISPVAL As Dword System
    #endIfMacro-
$endif

$if NumberDigits = 4
    #define __NumberDigits 4                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_STRING 5
    #define __LenghtDISP_STRING 5            '/ Assembler definition. For testing the variables.

    Dim DISP_STRING As String * LenghtDISP_STRING 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_STRING to avoid crashing the PIC
    Dim ONES As DISP_STRING#0
    Dim TENS As DISP_STRING#1
    Dim HUNDREDS As DISP_STRING#2
    Dim THOUSANDS As DISP_STRING#3

    Dim PRINT_ONES As Byte System        '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System        '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System    '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System   '/ Variable for interrupt routine.
    #ifMacro- _mEnterValue
    Dim DISPVAL As Word System
    #endIfMacro-
$endif
    
$if NumberDigits = 3
    #define __NumberDigits 3                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_STRING 4
    #define __LenghtDISP_STRING 4            '/ Assembler definition. For testing the variables..

    Dim DISP_STRING As String * LenghtDISP_STRING 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_STRING to avoid crashing the PIC
    Dim ONES As DISP_STRING#0
    Dim TENS As DISP_STRING#1
    Dim HUNDREDS As DISP_STRING#2

    Dim PRINT_ONES As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System
    Dim PRINT_HUNDREDS As Byte System
    #ifMacro- _mEnterValue
    Dim DISPVAL As Word System
    #endIfMacro-
$endif
    
'/TIMESHARE depends of number of bytes display.
    $if NumberDigits = 8
    Dim TIMESHARE As Word System    '/ Variable Used by the interrupt for time sharing
    $else
    Dim TIMESHARE As Byte System	'/ Variable Used by the interrupt for time sharing    
    $endif

$ifndef FSR0_Var
    $define FSR0_Var
	Dim FSR0 As FSR0L.Word         'Combine FSR0L\H into a single 16-bit register
$endif
$ifndef FSR1_Var
    $define FSR1_Var
	Dim FSR1 As FSR1L.Word         'Combine FSR1L\H into a single 16-bit register
$endif

'/ Global variables
    Dim dHighPointerFSR1 As Word System
    Dim dRsOutByteIn As Byte System 
    Dim dBytesReceived As Byte System 
    Dim dPrintErrorDelay As Word System
       
'/ Flags
    Dim dDISP_Flags As Byte System
    Dim F_PrintErrorDelay As dDISP_Flags.0 

'/ Temporal variables
    Dim dTempByte As Byte System
'/------------------------------------------------------------------------------
'/ Some special constants:
    $define MinusCharacter 191    '/%10111111 (-) value for Common Anode mode.

'/------------------------------------------------------------------------------
'/ Name          : Declare the different pins of the project.
'/ Input         : None
'/ Return Value  : None
'/ Parameters    : 
'/ Notes         : 
'/ 
    '/ Definition of the PORTS for Display segments.
    Symbol SEGMENTS = LATB              '/ PORT for Display segments = 8Bits

    '/ Definition of the pins to enable displays to write.
    Symbol PORT_Digits = LATA           '/ PORT to enable the digits
    '/ Pins location in the PORT: Always = LowNibble
    Symbol AddressDA = LATA.0           '/ Address 0 of the Digits.
    Symbol AddressDB = LATA.1           '/ Address 1 of the Digits.
    Symbol AddressDC = LATA.2           '/ Address 2 of the Digits.
    Symbol LE2 = LATA.3                 '/ Enable/Disable all Digits. 
    Symbol DipsAddressMask = %11110000  '/ LE2 = 0 + Valor to mask all bits of digits addresses.

    $ifdef _Keypad
    '/ Declare the position of the PORT Keypad LowNibble/HighNibble
    Symbol PORT_Keypad = PORTC          '/ PORT to read the Keypad
    '/ Pins location in the PORT. (LowNibble/HighNibble)
    $define _PORTkeypad LowNibble
    Symbol KA = PORTC.0                 '/ Input A Keypad
    Symbol KB = PORTC.1                 '/ Input B Keypad
    Symbol KC = PORTC.2                 '/ Input C Keypad
    Symbol KD = PORTC.3                 '/ Input D Keypad
    $endif

    '/ Declare the pin of the Buzzer. 
    $ifdef _BuzzerOption
    Symbol BUZZER = LATA.5              '/ Output Buzzer
    $endif
'/------------------------------------------------------------------------------
$ifdef _Keypad
    $if _PORTkeypad = LowNibble
    Symbol DipsKeypadMask = %00001111   '/ Enable Low nibble.
    Symbol NOKEYpressed = 15
    $else
    Symbol DipsKeypadMask = %11110000   '/ Enable High nibble.
    Symbol NOKEYpressed = 240    
    $endif
$endif
'/------------------------------------------------------------------------------
'/ Name          : Declare the different pins of the project.
'/ Input         : None
'/ Return Value  : None
'/ Purpose       : Variables & parameters for the Keypad option.
'/ Parameters    : 
'/ Notes         : 
'/ 
#ifMacro- _mInkeyDisp, _mEnterNumber, _mEnterValue, _mPasswordInput
    Dim dKeyPadFlags As Byte System
    Dim F_KeyPressed As dKeyPadFlags.0
    Dim F_KeyRead As dKeyPadFlags.1  
    Dim F_KeypadTimeOut As dKeyPadFlags.2
    Dim F_KeyBuzzerDelay As dKeyPadFlags.3
    Dim dKeyPressed As Byte System
    Dim dKeyPressedOld As Byte System
    Dim dMyKey As Byte System 
    Dim KeypadTimeOut As Word System
    Dim dKeyReleasedCounter As Byte System
    $ifdef _BuzzerOption
    Dim KeyBuzzerDelay As Byte System
    $endif
#endIfMacro-
'/------------------------------------------------------------------------------
'/ Name          : Declare the different pins of the project.
'/ Input         : None
'/ Return Value  : None
'/ Purpose       : Variables & parameters For the Password option.
'/ Parameters    : 
'/ Notes         : 
'/ 
$ifdef PassWordDigits
#ifMacro- _mPasswordInput
    $define LenghtKeypadString 9
    Dim dKeypadString As String * LenghtKeypadString
    Dim dNbPWDigits As Byte System 
    Dim dPassWordVal As Dword System
#endIfMacro-
$endif
'===============================================================================
'/ Name          : TableData
'/ Input         : ASCII data with binary format
'/ Return Value  : Segments lighted
'/ Parameters    : TableData = 0 for ROM Table. TableData = 1 For EEPROM Table.
'/ Notes         : Seven Segment data written For Commom Anode display.
'/                 Using some tricks it is not necessary using another table
'/                 for common cathode.
'/
$if TableData = 0
'/                       0   1   2   3   4   5   6   7   8   9   blank   blank   blank  blank      
SegmentsData: CData Byte 192,249,164,176,153,146,131,248,128,152,255,255,255,255,255,255,255,_
'/A  B   C   D   E   F   G   H   I   J   K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z   [   
136,131,198,161,134,142,144,137,207,225,138,199,200,171,163,140,148,175,146,135,227,193,213,137,145,164,255 '/ 44 numbers
'/ ":" is the character for Space on the display.
'/------------------------------------------------------------------------------
$else
'/------------------------------------------------------------------------------
'/ For more efficiency the Table EData must be placed in address 0 of the eeprom.
'/                       0   1   2   3   4   5   6   7   8   9  blank   blank   blank  blank
SegmentsData EData Byte 192,249,164,176,153,146,131,248,128,152,255,255,255,255,255,255,255,_
'/A  B   C   D   E   F   G   H   I   J   K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z   [   
136,131,198,161,134,142,144,137,207,225,138,199,200,171,163,140,148,175,146,135,227,193,213,137,145,164,255 '/ 44 numbers
'/ ":" is the character for Space on the display.
$endif
'===============================================================================
'===============================================================================
'/ Name          : Init the displays: DisplayInit()
'/ Input         : None
'/ Return Value  : COnfigure Timings and parameters of the project for one PIC only.
'/ Parameters    : 
'/ Notes         : Set TMR0 To Interrupt On rollover
'/                 The TMR0H & TMR0l counters are not loaded to simplify the code.
'/                 The PDS user must rewrite & check this routine for a new PIC.
'/
$define DisplayInit() GoSub DisplayInit_Sub

DisplayInit_Sub:
'/ CONFIGURATION OF TIMER0
'/ Set TMR0 to interrupt on rollover
'/ 
'/ could change in every Declare Xtal.
'/
    #ifdef watchdog_req
    Clrwdt
    #endif
    WDTCON = 0          '/ Without watchdog
    INTCON = 0                                              '/ Clear all interrupts
    While INTCONBits_GIE = 1 : INTCONBits_GIE = 0 : Wend	'/ Make sure global interrupts are off
'/##############################################################################
' Read the datasheet to change the FOSC frequency.
' The 8Mhz & 16Mhz outputs can use the PLL only.
'
    TBLPTRU = 0
$ifdef _InternalOSC_
    $if _xtal > 64
        $error "Error in (Declare Xtal) with internal oscillator"
    $endif
    ' See the Datasheet to configure the internal oscillator.
    '===========================================================================
INT_OSC:
    $if PLL_ConfigFuses = On
        $if _xtal = 64
        OSCCON = %01111000      ' 16Mhz x 4. Internal oscillator: Sleep, 16Mhz, R, R, Primary clock. 
                                ' Bit 1 = 0, Bit 0 = 0, Primary clock (determined by FOSC<3:0> in CONFIG1H).
                                ' bit 3 OSTS: Oscillator Start-up Time-out Status bit
                                ' 1 = Device is running from the clock defined by FOSC<3:0> of the CONFIG1H register
        OSCTUNEBits_PLLEN = 1   ' PLL enabled (FOSC = 4 x INTOSC)
        $endif        
        $if _xtal = 32
        OSCCON = %01101000      ' 8Mhz x 4. Internal oscillator: Sleep, 16Mhz, R, R, Primary clock. 
                                ' Bit 1 = 0, Bit 0 = 0, Primary clock (determined by FOSC<3:0> in CONFIG1H).
                                ' bit 3 OSTS: Oscillator Start-up Time-out Status bit
                                ' 1 = Device is running from the clock defined by FOSC<3:0> of the CONFIG1H register
        OSCTUNEBits_PLLEN = 1   ' PLL enabled (FOSC = 4 x INTOSC)
        $endif
    $endif 
    $if PLL_ConfigFuses = OFF
        $if _xtal = 16
        OSCCON = %01111000      ' 16Mhz x 1. Internal oscillator: Sleep, 16Mhz, R, R, Primary clock Direct. 
                                ' Bit 1 = 0, Bit 0 = 0, Primary clock (determined by FOSC<3:0> in CONFIG1H).
                                ' bit 3 OSTS: Oscillator Start-up Time-out Status bit
                                ' 1 = Device is running from the clock defined by FOSC<3:0> of the CONFIG1H register
        OSCTUNEBits_PLLEN = 0   ' PLL disabled (FOSC = 1 x INTOSC direct)
        $endif
    $endif
    '---------------------------------------------------------------------------
    OSCTUNEBits_INTSRC = 1              ' 1 = 31.25 kHz device clock derived from 16 MHz HFINTOSC source (divide-by-512 enabled) (if necessary)
    '---------------------------------------------------------------------------       
    $ifdef OSCCONBits_IOFS
    While OSCCONBits_IOFS = 0 : Wend    ' Wait untill OSC is stable (not a good job)
    $else
        $ifdef OSCCONBits_HFIOFS
        While OSCCONBits_HFIOFS = 0 : Wend  ' Wait untill OSC is stable (not a good job)
        $endif
    $endif    
    DelayMS 25
    '===========================================================================
$else
    '===========================================================================
    ' Oscillator Start-up Time-out Status bit:
    ' Device is running from the clock defined by FOSC<2:0> of the CONFIG1 register
    ' System Clock Select bits:
    ' Device is running from the clock defined by FOSC<3:0> of the CONFIG1H register
    OSCCON = %01111000       
    DelayMS 12
$endif    
'/##############################################################################
    INTCON2 = 0
    INTCON2Bits_RBPU = 1        '/ All PORTB pull-ups are disabled.
    RCONBits_IPEN = 1           '/ Enable High Priority levels on interrupts only.
'/------------------------------------------------------------------------------
'/ Code to configure the Timer0.
$ifdef ScanTimer0
    Asm
    messg "CONFIG TIMER0"
    EndAsm
    INTCON2Bits_TMR0IP = 1      '/ Timer0 for High Interrupt.

    '/ Configure the Timer0 for FOSC = 4Mhz.
    $if _xtal = 4
    T0CON = %00000100           '/ div32 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $define _TimingMS 8
    $endif

    '/ Configure the Timer0 for FOSC = 8Mhz used for Proteus.
    $if NumberDigits = 8 Or NumberDigits = 7
        $if _xtal = 8
        T0CON = %00000100           '/ div32 => 4,0 mS Timer0 Prescaler Select bits 
        T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
        T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
        T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
        $define _TimingMS 4
        $endif
    $else
        $if _xtal = 8
        T0CON = %00000101           '/ div64 => 8,1 mS Timer0 Prescaler Select bits 
        T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
        T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
        T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
        $define _TimingMS 8
        $endif   
    $endif

    '/ Configure the Timer0 for FOSC = 16Mhz.
    $if _xtal = 16
    T0CON = %00000110           '/ div128 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $define _TimingMS 8
    $endif

    '/ Configure the Timer0 for FOSC = 20Mhz.
    $if _xtal = 20
    T0CON = %00000110           '/ div128 => 6,55 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $define _TimingMS 7
    $endif

    '/ Configure the Timer0 for FOSC = 32Mhz.
    $if _xtal = 32
    T0CON = %00000111           '/ div256 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $define _TimingMS 8
    $endif

    '/ Configure the Timer0 for FOSC = 40Mhz.
    $if _xtal = 40
    T0CON = %00000000           '/ div2 => 13,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $define _TimingMS 13
    $endif

    '/ Configure the Timer0 for FOSC = 64Mhz for Kxx PICs.
    $if _xtal = 64
    T0CON = %00000001           '/ div4 => 8,2 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $define _TimingMS 8
    $endif

    '/ Configure the Timer0 for FOSC = 80Mhz for K20 Pics.
    $if _xtal = 80
    T0CON = %00000000           '/ div2 => 6,55 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $define _TimingMS 7
    $endif

    TMR0H = 0                   '/ Clear the Timer0 counter.
    Nop
    TMR0L = 0
    Nop    
    T0CONBits_TMR0ON = 1        '/ Start the Timer0.
$endif  '/ ScanTimer0
'/------------------------------------------------------------------------------
'/ Code to configure the Timer2.
$ifdef ScanTimer2
    Asm
    messg "CONFIG TIMER2"
    EndAsm
    IPR1Bits_TMR2IP = 1         '/ Timer2 for High Interrupt.

    $if _xtal = 4
        ' TMR2 8Bits counter, Postscaler 1/10, Prescaler 1/4, Clock Xtal/4 (Xtal: Xtal 4 Mhz)
        ' For Timer2 interrupt every 10,0 mS 
        T2CON = %01001001
        PR2 = 248
        $define _TimingMS 10
    $endif

    '/ used For Proteus
    $if NumberDigits = 8 Or NumberDigits = 7
        $if _xtal = 8
            ' TMR2 8Bits counter, Postscaler 1/3, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 8 Mhz)
            ' For Timer2 interrupt every 6,0 mS 
            T2CON = %00010011
            PR2 = 249
            $define _TimingMS 6
        $endif
    $else
        $if _xtal = 8
            ' TMR2 8Bits counter, Postscaler 1/5, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 8 Mhz)
            ' For Timer2 interrupt every 8,0 mS 
            T2CON = %00100011
            PR2 = 249
            $define _TimingMS 8
        $endif
    $endif

    $if _xtal = 16
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 4 + PLL = 16 Mhz) or (Xtal: Xtal 16Mhz)
        ' For Timer2 interrupt every 10,0 mS 
        T2CON = %01111011
        PR2 = 155        
        $define _TimingMS 10
    $endif
    $if _xtal = 20
        ' TMR2 8Bits counter, Postscaler 1/13, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz)
        ' For Timer2 interrupt every 10,00 mS 
        T2CON = %01100011
        PR2 = 239
        $define _TimingMS 10
    $endif
    $if _xtal = 32
        ' TMR2 8Bits counter, Postscaler 1/4, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz)
        ' For Timer2 interrupt every 8,2 mS 
        T2CON = %01111011
        PR2 = 255
        $define _TimingMS 8
    $endif
    $if _xtal = 40
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz) or (Xtal: Xtal 10 Mhz + PLL)
        ' For Timer2 interrupt every 6,6 mS 
        T2CON = %01111011
        PR2 = 255
        $define _TimingMS 6
    $endif
    $if _xtal = 64
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Osc Off, Clock Xtal/4 (Xtal: Xtal 16 + PLL = 64Mhz)
        ' For Timer2 interrupt every 4,2 mS
        T2CON = %01111011
        PR2 = 255
        $define _TimingMS 4
    $endif

    ' Updated for Xtal = 80 Mhz, but for testing only because the Timing is only 3,2 mS.
    ' For K20 series.
    $if _xtal = 80
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Osc Off, Clock Xtal/4 (Xtal: Xtal 20 + PLL = 80Mhz)
        ' For Timer2 interrupt every 3,3 mS
        T2CON = %01111011
        PR2 = 255
        $define _TimingMS 4
    $endif       
       
    TMR2 = 0                    '/ Clear the Timer2 counter.
    Nop    
    T2CONBits_TMR2ON = 1        '/ Start the Timer2.
$endif  '/ ScanTimer2
'/------------------------------------------------------------------------------
    '/ The true Error Delay:
    Symbol VErrorDelayMS = ErrorDelayMS/_TimingMS

    '/ The true Keypad TimeOut:
    $ifdef KeypadTimeoutMS 
    Symbol VKeypadTimeout = KeypadTimeoutMS/_TimingMS
    $endif

    '/ The true Buzzer Delay:
    $ifdef _BuzzerOption
    Symbol VKeyBuzzerDelay = BuzzerDelayMS/_TimingMS
    $endif
'/------------------------------------------------------------------------------
    Asm
    messg "CONFIG PARAMETERS"
    EndAsm
    '/ Set the PORTS to digital.
    ADCON0 = 0    
    CM1CON0 = 0
    CM2CON0 = 0
    CM2CON1 = 0
    ANSEL = 0
    ANSELH = 0

	Output SEGMENTS             '/ Put the display PORT as Output.

    ' Blanks directly the Port at reset depending of the option.
    $if DisplayMode = 0
    SEGMENTS = 0                '/ All segments cleared for common cathode.
    $else
    SEGMENTS = 255              '/ All segments cleared for common anode.
    $endif

    ' Write the lines OFF at reset.
    $ifdef _Keypad
    Input KA        '/ Input A Keypad
    Input KB        '/ Input B Keypad
    Input KC        '/ Input C Keypad
    Input KD        '/ Input D Keypad
    $endif    

    High LE2        '/ All digits off.
    Low AddressDA
    Low AddressDB
    Low AddressDC
    PORT_Digits = 0
    $ifdef _BuzzerOption
    Low BUZZER
    $endif
	TIMESHARE = 1		        '/ Reset the time share variable
    dDISP_Flags = 0 

    '/ Initialisation of the Keypad.
    $ifdef _Keypad
    dKeyPadFlags = 0
    dKeyPressed = NOKEYpressed
    dKeyPressedOld = NOKEYpressed
    dKeyReleasedCounter = 0
    $endif

    '/ Blanks the Displays at reset.
    $if NumberDigits = 8
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 255   '/ Blank the THOUSANDTHOUSANDS DIGIT
    PRINT_TENTHDTHOUSANDS = 255     '/ Blank the TENTHDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 7
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 255   '/ Blank the THOUSANDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 6
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 5
    PRINT_ONES = 255            '/ Blank the ONES DIGIT
    PRINT_TENS = 255            '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255        '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255       '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255    '/ Blank the TENTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 4
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255    '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255   '/ Blank the THOUSANDS DIGIT
    $endif
    $if NumberDigits = 3
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255    '/ Blank the HUNDREDS DIGIT
    $endif

    Clear DispDummyArray    '/ Not very important.

    '/ Set all parameters of the array to work.
    GoSub RestoreArrayParameters_Sub

'/ Code to configure the Timer0.
$ifdef ScanTimer0
    Asm
    messg "CONFIG TIMER0"
    EndAsm
    ' Configure the interrupts to start.
    INTCONBits_TMR0IF = 0   '/ Clear the Timer0 interrupt flag to begin.
    INTCONBits_TMR0IE = 1   '/ Enable the Timer0 interrupts.
	INTCONBits_GIE = 1	    '/ Enables all High priority interrupts
$endif

'/ Code to configure the Timer2.
$ifdef ScanTimer2
    Asm
    messg "CONFIG TIMER2"
    EndAsm
    ' Configure the interrupts to start.
    PIR1Bits_TMR2IF = 0     '/ Clear the Timer0 interrupt flag to begin.
    PIE1Bits_TMR2IE = 1     '/ Enable the Timer0 interrupts.
	INTCONBits_GIE = 1	    '/ Enables all High priority interrupts
$endif
    ' The Timer is running...
    Return
'===============================================================================
'===============================================================================
'/ Name          : InkeyD() 
'/ Input         : None
'/ Return Value  : Return_Var (dMyKey).
'/ Purpose       : Macro to read the Keypad.
'/ Parameters    : A lookdown table is used to recognize the keys.
'/ Notes         : The Values of ROW and Column are read by interrupts.
'/               : If Macro is not written in main code, this code code is not compiled.
'/
'/ To read the Keypad.
$define InkeyD() _mInkeyDisp

_mInkeyDisp Macro-
    Asm
    messg "Macro - InkeyD()" '/ To find this macro in the asm file.
    EndAsm
    GoSub InkeyDisp_Sub
    #if (_mInkeyDisp_RETURN != 1)
        #error "_mInkeyDisp - Mandatory return parameter misssing"
        Exitm
    #else
        #if(Return_Type != Byte)
            #error "_mInkeyDisp - Macro - Return variable should be a Byte variable"
            Exitm
        #else
            Byte_Byte dMyKey,Return_Var
        #endif
    #endif
Endm

#ifMacro- _mInkeyDisp, _mEnterNumber, _mEnterValue, _mPasswordInput
InkeyDisp_Sub:
    Asm
    messg "Macro - InkeyD() subroutine" '/ To find this routine in the asm file.
    EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif

    dMyKey = 255                        '/ Do not repeat the last key.
    If F_KeyRead = 0 Then
        If F_KeyPressed = 1 Then
            '/ Delay for Buzzer when a Key is pressed
            $ifdef _BuzzerOption
            F_KeyBuzzerDelay = 1                            '/ Enable the Buzzer pulse.
            '/ This delay must to be adjusted with the Interrupt Timing.
            KeyBuzzerDelay = VKeyBuzzerDelay                '/ Time of the Buzzer pulse.
            High BUZZER                                     '/ Start Buzzer.
            $endif

            If dKeyPressed <> dKeyPressedOld Then
                '/--------------------------------------------------------------
                '/ Here place the Keypad table data.
                '/ Using a LookDown command, the returned values can be re-arranged
                '/ to correspond with the legends printed on the keypad:
                $if _Keypad = 4X4
                '/KEY                    0         1         2         3
                dMyKey = LookDown dKeyPressed,[%01110010,%10110001,%10110010,%10110100,_
                '/4        5         6         7         8         9         % 10
                %11010001,%11010010,%11010100,%11100001,%11100010,%11100100,%11101000,_
                '/X 11     - 12      + 13      = 14      ON 15
                %11011000,%10111000,%01111000,%01110100,%01110001]
                $define KeypadExit 15
                $endif

                $if _Keypad = 4X3
                '/KEY                    0         1         2         3
                dMyKey = LookDown dKeyPressed,[%01110010,%11100001,%11100010,%11100100,_
                '/4        5         6         7         8         9         * 10      # 11
                %11010001,%11010010,%11010100,%10110001,%10110010,%10110100,%01110001,%01110100]
                $define KeypadExit 11
                $endif                
                '/--------------------------------------------------------------
                dKeyPressedOld = dKeyPressed
                F_KeyRead = 1               '/ Key read OK, waiting key depressed.
            EndIf
            F_KeyPressed = 0                '/ The present key is read, waiting the next key.
        EndIf
    Else
        '/ NO auto-repeat code.
        F_KeyPressed = 0                    '/ Clear key pressed until key released.
        DelayMS 5                           '/ This Delay could be adjusted. 
        '/ Debounce code for key released.
        If dKeyPressed = NOKEYpressed Then  '/ Check if Key is released.
            Inc dKeyReleasedCounter
            If dKeyReleasedCounter = 10 Then
                dKeyReleasedCounter = 0         '/ Re-init the debounce counter.
                dKeyPressedOld = NOKEYpressed   '/ Then reinit the Key.
                F_KeyRead = 0                   '/ Read again the Keypad.
            EndIf
        Else
            dKeyReleasedCounter = 0         '/ If different key then count again.
        EndIf
    EndIf 
    Return
#endIfMacro-
'===============================================================================
'/ Name          : EnterNumber(pValue,pExit): Macro To enter a number with the keypad.
'/ Input         : pValue = (Byte, Word Or Dword), pExit Variable (Byte) used For error Output.
'/ Return Value  : Return_Var (dMyKey).
'/ Parameters    : A lookdown table is used to recognize the keys.
'/ Notes         : This Code is an infinite Loop. The exit is done by typing a exit key On the keypad (KeypadExit).
'/               : pValue = Byte, Word or Dword
'/               : pExit = Byte
'/                 The output variable (pValue) is cleared before to use.
'/                 pExit is a variable (Byte) used for error output. If = 0 then error. If = 255 then continue...
'/               : If Macro is not written in main code, this code code is not compiled.
'/
$define EnterNumber(pValue,pExit)           '
    #ifdef watchdog_req                     '
    Clrwdt                                  '
    #endif                                  '
    _mEnterNumber pValue,pExit              '
    KeypadTimeOut = VKeypadTimeout          '
    F_KeypadTimeOut = 1                     '
    pValue = 0                              '
    pExit = 0                               '
    F_KeyPressed = 0                        '
    dKeyPressed = NOKEYpressed              '
    Do                                      '
        GoSub InkeyDisp_Sub                 '
        If dMyKey = KeypadExit Then         '
            KeypadTimeOut = VKeypadTimeout  '
            pExit = 255                     '
            GoTo EnterNumberExit            '
        ElseIf dMyKey < 10 Then             '
            KeypadTimeOut = VKeypadTimeout  '
            pValue = pValue * 10            '
            pValue = pValue + dMyKey        '
            RsOut Dec pValue,CR             '
            If F_PrintErrorDelay = 1 Then   '
                Break                       '
            EndIf                           '
        EndIf                               '
        If F_KeypadTimeOut = 0 Then         '
            Break                           '
        EndIf                               '
    Loop                                    '
    pValue = 0                              '
EnterNumberExit:                            '
    RsOut ":",CR


'/ This macro is used for testing variables only.
_mEnterNumber Macro- pValue,pExit
    Asm
    messg "Macro - EnterNumber()"
    EndAsm
    #if (Prm_Count != 2)
        #error "_mEnterNumber - Macro - needs 2 parameters!"
        Exitm
    #else
        #if (Prm_2 != Byte)
            #error "_mEnterNumber Macro - (Param2) Must be a Byte Variable"
            Exitm
        #endif
    #endif
Endm
'/------------------------------------------------------------------------------
'/ Name          : EnterValue(pValue,pExit): Macro To enter a number with the keypad with subroutine.
'/ Input         : pValue = (Byte, Word Or Dword), pExit Variable (Byte) used For error Output.
'/ Return Value  : Return_Var (dMyKey).
'/ Parameters    : A lookdown table is used to recognize the keys.
'/ Notes         : This Code is an infinite Loop. The exit is done by typing a exit key On the keypad (KeypadExit).
'/               : pValue = Byte, Word or Dword
'/               : pExit = Byte
'/                 The output variable (pValue) is cleared before to use.
'/                 pExit is a variable (Byte) used for error output. If = 0 then error. If = 255 then continue...
'/                 DISPVAL variable is defined by the number of digits.
'/               : If Macro is not written in main code, this code code is not compiled.
'/ 
$define EnterValue(pValue,pExit) '
    Asm                          '
    messg "Macro - EnterValue()" '
    EndAsm                       '
    _mEnterValue pValue,pExit    '
    pExit = dTempByte            '
    pValue = DISPVAL

'/ Macro explanation:
'/ pValue is the Output variable defined by the user. This variable is cleared before to use.
'/ pExit is the control byte to return an error or a valid Value.
'/
'/ This macro is used for testing variables only.
_mEnterValue Macro- pValue,pExit
    #if (Prm_Count != 2)
        #error "_mEnterValue - Macro - needs 2 parameters!"
        Exitm
    #else
        #if (Prm_2 != Byte)
            #error "_mEnterValue Macro - (Param2) Must be a Byte Variable"
            Exitm
        #endif
    #endif
    GoSub _mEnterValue_Sub
Endm

#ifMacro- _mEnterValue
_mEnterValue_Sub:
    Asm
    messg "Macro - EnterValue() Subroutine"
    EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif
    dTempByte = 0                               '/ Character of control.  
    DISPVAL = 0                                 '/ The variable is cleared before to use.
    KeypadTimeOut = VKeypadTimeout              '/ Load the Time Out value
    F_KeypadTimeOut = 1                         '/ Start the TimeOut
    F_KeyPressed = 0                            '/ Clear a key pressed before.
    dKeyPressed = NOKEYpressed                  '/ Clear a key pressed before.
    Do
        GoSub InkeyDisp_Sub     '/ Read the Keypad. The result is in dMyKey variable.

        Select dMyKey
            '/ Check for the exit key.
            Case KeypadExit
                KeypadTimeOut = VKeypadTimeout      '/ Reload the TimeOut value when every key is pressed.
                dTempByte = 255                     '/ Successfull exit.
                GoTo _mEnterValueExit               '/ Return a valid value.

            '/ Check for a valid key.
            Case < 10
                KeypadTimeOut = VKeypadTimeout      '/ Reload the TimeOut value when every key is pressed.
                DISPVAL = DISPVAL * 10
                DISPVAL = DISPVAL + dMyKey          '/ Add a new digit from keypad
                RsOut Dec DISPVAL,CR                '/ If not a password option then display the value.            
                If F_PrintErrorDelay = 1 Then
                    Break                           '/ If error then exit the loop.
                EndIf
        EndSelect
       
        '/ Check for the timeout.
        If F_KeypadTimeOut = 0 Then
            Break                           '/ If TimeOut expired then exit the loop.
        EndIf
    Loop
    DISPVAL = 0         '/ If error or timeout clear the exit value.
_mEnterValueExit:
    RsOut ":",CR        '/ Clear the display.
    Return              '/ Exit the EnterValue() command.
#endIfMacro-
'/------------------------------------------------------------------------------
'/ Name          : PasswordInput(pValue,pExit) 
'/ Input         : pValue is the Variable (Dword) To be loaded with the keypad.
'/ Return Value  : pExit (Byte) for control, dPassWordVal (Dword) value.
'/ Purpose       : Macro To enter a Password with the keypad.
'/ Parameters    : NumberDigits / VKeypadTimeout / _TimingMS
'/ Notes         : It is the same command As "EnterValue(pValue,pExit)".
'/               : dPassWordVal Variable is defined As a Dword.
'/               : pExit is a Variable (Byte) used For error Output. If = 0 Then error. If = 255 Then a valid value.
'/
'/               : If Macro is not written in main code, this code code is not compiled.
$define PasswordInput(pValue,pExit) '
    Asm                             '
    messg "Macro - PasswordInput()" '
    EndAsm                          '
    _mPasswordInput pValue,pExit    '
    pExit = dTempByte               '
    pValue = dPassWordVal

'/ Macro explanation:
'/ pValue is the Output variable defined by the user. This variable is cleared before to use.
'/ pExit is the control byte to return an error or a valid Value.
'/
'/ This macro is used for testing variables only.
_mPasswordInput Macro- pValue,pExit
    #if (Prm_Count != 2)
        #error "_mPasswordInput - Macro - needs 2 parameters!"
        Exitm
    #else
        #if (Prm_2 != Byte)
            #error "_mPasswordInput Macro - (Param2) Must be a Byte Variable"
            Exitm
        #endif
    #endif
    GoSub PasswordInput_Sub
Endm

#ifMacro- _mPasswordInput
PasswordInput_Sub:
    Asm
    messg "Macro - _mPasswordInput() Subroutine"
    EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif
    dTempByte = 0                               '/ Init the control character.  
    dPassWordVal = 0                            '/ The output variable is cleared before to use.
    dNbPWDigits = 0                             '/ Init the digits counter.
    Clear dKeypadString                         '/ Clear all bytes of this string.
    KeypadTimeOut = VKeypadTimeout              '/ Load the TimeOut value
    F_KeypadTimeOut = 1                         '/ Start the TimeOut by interrupt routine.
    F_KeyPressed = 0                            '/ Clear a key pressed before.
    dKeyPressed = NOKEYpressed                  '/ Clear a key pressed before.
    Do
        GoSub InkeyDisp_Sub     '/ Read the Keypad. The result is in dMyKey variable.

        If dMyKey < KeypadExit + 1 Then
            KeypadTimeOut = VKeypadTimeout              '/ Reload the TimeOut value when every key is pressed.
            Inc dNbPWDigits                             '/ Increment number of digits for every key pressed. 
            '/ Check for a valid key.
            If dMyKey < 10 Then                         '/ Choose numeric keys only.
                dPassWordVal = dPassWordVal * 10        '/ Move 1 decimal digit to the left.
                dPassWordVal = dPassWordVal + dMyKey    '/ Add a new digit from keypad
            Else
                dPassWordVal = 0                        '/ If not a valid key then clear the password.
            EndIf
            '/ All keys print a minus (-).
            WREG = Len (dKeypadString)          '/ Check how many (-) in the string.
            If WREG = NumberDigits Then         '/ Do not overrun the display.
                Clear dKeypadString             '/ The String is empty.
            EndIf
            dKeypadString = dKeypadString + "-" '/ For Password option do not print the value of the variable. 
            RsOut Str dKeypadString,CR          '/ Print a "-" to the display every time a key is pressed.
            If F_PrintErrorDelay = 1 Then
                Break                           '/ If error then exit the loop.
            EndIf
        EndIf       

        '/ Check the number of keys pressed.
        If dNbPWDigits = PassWordDigits Then
            dTempByte = 255                 '/ Successfull exit.
            GoTo PasswordEntryExit          '/ Return a valid value.   
        EndIf

        '/ Check for the timeout.
        If F_KeypadTimeOut = 0 Then
            Break                           '/ If TimeOut expired then exit the loop.
        EndIf
    Loop
    dPassWordVal = 0    '/ If error or timeout clear the exit value.
PasswordEntryExit:
    Return              '/ Exit the PasswordInput() command.
#endIfMacro-
'===============================================================================
'===============================================================================
'/ Name          : Interrupt HANDLER
'/ Input         : 
'/ Return Value  : SEGMENTS, KEY pressed
'/ Parameters    : 
'/ Notes         : Multiplexing the 8-digits
'/
MULTILEX_DIGITS:
    Asm
    messg "INTERRUPT ROUTINE" ;/ To find this routine in the asm file.
    EndAsm
Context Save				

    If DisPlayInterruptFlag = 1 Then 		
        '/ Process the first display
        Asm
        messg "DIGIT ONE" ;/ To find this routine in the asm file.
        EndAsm
        If TIMESHARE.0 = 1 Then             '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF.  
            $ifdef _ProteusTest
            DelayCS 2                       '/ Problem of Proteus. Could be disabled for real wold.
            $endif
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_ONES          '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            DelayCS 1                       '/ Problem of Proteus. Must be the same execution time to write the PORT.
            SEGMENTS = PRINT_ONES           '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            PORT_Digits = PORT_Digits & DipsAddressMask  '/ Read + clear all bits of Digits address + Multiplexer On.
                                                         '/ Write DIGIT number 1
            '/ Keypad option.
            $ifdef _Keypad
            '/ Read the Keypad column 1.                          
            If F_KeyPressed = 0 Then
                '/ Read the 4 Rows
                $if DisplayMode = 0
                DelayCS 1
                dKeyPressed = PORT_Keypad & DipsKeypadMask
                $else
                DelayCS 2                   '/ Must be the same execution time to read the PORT.
                WREG = ~PORT_Keypad
                dKeyPressed = WREG & DipsKeypadMask
                $endif
                '/ Check if KEY pressed.
                If dKeyPressed < NOKEYpressed Then  '/ Check if Key pressed.
                    $if _PORTkeypad = LowNibble
                    Asm
                    Swapf dKeyPressed,W,0           ;/ Swap the 2 nibbles of the dKeyPressed variable.
                    EndAsm
                    dKeyPressed = WREG | TIMESHARE          '/ 4 Rows + 1 Column
                    $else
                    dKeyPressed = dKeyPressed | TIMESHARE   '/ 4 Rows + 1 Column
                    $endif
                    F_KeyPressed = 1        '/ Stop reading the PORT, waiting InkeyD command.
                EndIf
            EndIf
            $endif
            
        '/ Process the second display									
        Asm
        messg "DIGIT TWO" ;/ To find this routine in the asm file.
        EndAsm
        ElseIf TIMESHARE.1 = 1 Then         '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF. 
            $ifdef _ProteusTest
            DelayCS 2                       '/ Problem of Proteus. Could be disabled for real wold.
            $endif
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENS          '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            DelayCS 1                       '/ Problem of Proteus. Must be the same execution time to write the PORT.
            SEGMENTS = PRINT_TENS           '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask  '/ Read + Clear all bits of Digits address + Multiplexer On.
            PORT_Digits = WREG | 1                '/ Write DIGIT number 2

            '/ Keypad option.
            $ifdef _Keypad
            '/ Read the Keypad column 2.                          
            If F_KeyPressed = 0 Then
                '/ Read the 4 Rows
                $if DisplayMode = 0
                DelayCS 1
                dKeyPressed = PORT_Keypad & DipsKeypadMask
                $else
                DelayCS 2                   '/ Must be the same execution time To read the PORT.
                WREG = ~PORT_Keypad
                dKeyPressed = WREG & DipsKeypadMask
                $endif
                '/ Check if KEY pressed.
                If dKeyPressed < NOKEYpressed Then   '/ Check if Key pressed.
                    $if _PORTkeypad = LowNibble
                    Asm
                    Swapf dKeyPressed,W,0            ;/ Swap the 2 nibbles of the dKeyPressed variable.
                    EndAsm
                    dKeyPressed = WREG | TIMESHARE          '/ 4 Rows + 1 Column
                    $else
                    dKeyPressed = dKeyPressed | TIMESHARE   '/ 4 Rows + 1 Column
                    $endif
                    F_KeyPressed = 1        '/ Stop reading the PORT, waiting InkeyD command.    
                EndIf
            EndIf
            $endif
            
        '/ Process the third display									
        Asm
        messg "DIGIT THREE" ;/ To find this routine in the asm file.
        EndAsm
        ElseIf TIMESHARE.2 = 1 Then         '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF. 
            $ifdef _ProteusTest
            DelayCS 2                       '/ Problem of Proteus. Could be disabled for real wold.
            $endif
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_HUNDREDS      '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            DelayCS 1                       '/ Problem of Proteus. Must be the same execution time to write the PORT.
            SEGMENTS = PRINT_HUNDREDS       '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask  '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 2                '/ Write DIGIT number 3

            '/ Keypad option.
            $ifdef _Keypad
            '/ Read the Keypad column 3.                          
            If F_KeyPressed = 0 Then
                '/ Read the 4 Rows
                $if DisplayMode = 0
                DelayCS 1
                dKeyPressed = PORT_Keypad & DipsKeypadMask
                $else
                DelayCS 2                   '/  Must be the same execution time To read the PORT.
                WREG = ~PORT_Keypad
                dKeyPressed = WREG & DipsKeypadMask
                $endif
                '/ Check if KEY pressed.
                If dKeyPressed < NOKEYpressed Then   '/ Check if Key pressed.
                    $if _PORTkeypad = LowNibble
                    Asm
                    Swapf dKeyPressed,W,0            ;/ Swap the 2 nibbles of the dKeyPressed variable.
                    EndAsm                    
                    dKeyPressed = WREG | TIMESHARE          '/ 4 Rows + 1 Column
                    $else
                    dKeyPressed = dKeyPressed | TIMESHARE   '/ 4 Rows + 1 Column
                    $endif
                    F_KeyPressed = 1        '/ Stop reading the PORT, waiting InkeyD command.  
                EndIf
            EndIf
            $endif
                
        '/ Process the fourth display			
        $if NumberDigits > 3
        Asm
        messg "DIGIT FOUR" ;/ To find this routine in the asm file.
        EndAsm
        ElseIf TIMESHARE.3 = 1 Then         '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF. 
            $ifdef _ProteusTest
            DelayCS 2                       '/ Problem of Proteus. Could be disabled for real wold.
            $endif
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_THOUSANDS     '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            DelayCS 1                       '/ Problem of Proteus. Must be the same execution time to write the PORT.
            SEGMENTS = PRINT_THOUSANDS      '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask  '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 3                '/ Write DIGIT number 4

            '/ Keypad option.
            $ifdef _Keypad
            '/ Read the Keypad column 4.                          
            If F_KeyPressed = 0 Then
                '/ Read the 4 Rows
                $if DisplayMode = 0
                DelayCS 1
                dKeyPressed = PORT_Keypad & DipsKeypadMask
                $else
                DelayCS 2                   '/ Must be the same execution time To read the PORT.
                WREG = ~PORT_Keypad
                dKeyPressed = WREG & DipsKeypadMask
                $endif
                '/ Check if KEY pressed.
                If dKeyPressed < NOKEYpressed Then  '/ Check if Key pressed.
                    $if _PORTkeypad = LowNibble
                    Asm
                    Swapf dKeyPressed,W,0           ;/ Swap the 2 nibbles of the dKeyPressed variable.
                    EndAsm
                    dKeyPressed = WREG | TIMESHARE          '/ 4 Rows + 1 Column
                    $else
                    dKeyPressed = dKeyPressed | TIMESHARE   '/ 4 Rows + 1 Column
                    $endif
                    F_KeyPressed = 1        '/ Stop reading the PORT, waiting InkeyD command.    
                EndIf
            EndIf
            $endif
        $endif

        '/ Process the fifht display			
        $if NumberDigits > 4
        Asm
        messg "DIGIT FIVE" ;/ To find this routine in the asm file.
        EndAsm
        ElseIf TIMESHARE.4 = 1 Then         '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF. 
            $ifdef _ProteusTest
            DelayCS 2                       '/ Problem of Proteus. Could be disabled for real wold.
            $endif
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENTHOUSANDS  '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            DelayCS 1                       '/ Problem of Proteus. Must be the same execution time to write the PORT.
            SEGMENTS = PRINT_TENTHOUSANDS   '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask  '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 4                '/ Write DIGIT number 5
            '/ Place here the same Keypad code for more keys.
        $endif

        '/ Process the sixfht display			
        $if NumberDigits > 5
        Asm
        messg "DIGIT SIX" ;/ To find this routine in the asm file.
        EndAsm
        ElseIf TIMESHARE.5 = 1 Then             '/ If it's our turn then...
            LE2 = 1                             '/ All digits OFF. 
            $ifdef _ProteusTest
            DelayCS 2                           '/ Problem of Proteus. Could be disabled for real wold.
            $endif
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_HUNDREDTHOUSANDS  '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            DelayCS 1                           '/ Problem of Proteus. Must be the same execution time to write the PORT.
            SEGMENTS = PRINT_HUNDREDTHOUSANDS   '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask    '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 5                  '/ Write DIGIT number 6
            '/ Place here the same Keypad code for more keys.
        $endif

        '/ Process the sevenfht display			
        $if NumberDigits > 6
        Asm
        messg "DIGIT SEVEN" ;/ To find this routine in the asm file.
        EndAsm
        ElseIf TIMESHARE.6 = 1 Then             '/ If it's our turn then...
            LE2 = 1                             '/ All digits OFF. 
            $ifdef _ProteusTest
            DelayCS 2                           '/ Problem of Proteus. Could be disabled for real wold.
            $endif
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_THOUSANDTHOUSANDS '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            DelayCS 1                           '/ Problem of Proteus. Must be the same execution time to write the PORT.
            SEGMENTS = PRINT_THOUSANDTHOUSANDS  '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask    '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 6                  '/ Write DIGIT number 7
            '/ Place here the same Keypad code for more keys.
        $endif

        '/ Process the eightfht display			
        $if NumberDigits > 7
        Asm
        messg "DIGIT EIGHT" ;/ To find this routine in the asm file.
        EndAsm
        ElseIf TIMESHARE.7 = 1 Then             '/ If it's our turn then...
            LE2 = 1                             '/ All digits OFF. 
            $ifdef _ProteusTest
            DelayCS 2                           '/ Problem of Proteus. Could be disabled for real wold.
            $endif
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENTHDTHOUSANDS   '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            DelayCS 1                           '/ Problem of Proteus. Must be the same execution time to write the PORT.
            SEGMENTS = PRINT_TENTHDTHOUSANDS    '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask    '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 7                  '/ Write DIGIT number 8
            '/ Place here the same Keypad code for more keys.
        $endif

        EndIf
        
        Asm
        messg "TIME SHARING" ;/ To find this routine in the asm file.
        EndAsm
        '/ Process the scanning.
        TIMESHARE = TIMESHARE << 1      '/ Shift bit to set the next digit display.

        '/ Checking the last digit & reset scanning.        
        $if NumberDigits = 8
        If TIMESHARE.8 = 1 Then			'/ Is Bit 8 Set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 7
        If TIMESHARE.7 = 1 Then			'/ Is bit 7 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 6
        If TIMESHARE.6 = 1 Then			'/ Is bit 6 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 5
        If TIMESHARE.5 = 1 Then			'/ Is bit 5 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 4
        If TIMESHARE.4 = 1 Then			'/ Is bit 4 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 3
        If TIMESHARE.3 = 1 Then			'/ Is bit 3 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif        
        DisPlayInterruptFlag = 0        '/ Reset TMR0 interrupt flag

        '/ Keypad option.
        $ifdef _Keypad
        Asm
        messg "KEYPAD OPTION" ;/ To find this routine in the asm file.
        EndAsm
        '/ Security, Timeout for the keypad
        If F_KeypadTimeOut = 1 Then
            Dec KeypadTimeOut
            If KeypadTimeOut = 0 Then
                F_KeypadTimeOut = 0
            EndIf 
        EndIf

        $ifdef _BuzzerOption
        Asm
        messg "BUZZER OPTION" ;/ To find this routine in the asm file.
        EndAsm
        '/ Delay for Buzzer when a Key is pressed
        If F_KeyBuzzerDelay = 1 Then
            Dec KeyBuzzerDelay
            If KeyBuzzerDelay = 0 Then
                F_KeyBuzzerDelay = 0
                Low BUZZER
            EndIf 
        EndIf
        $endif        
        $endif

        '/ Security, decrement an error delay
        Asm
        messg "ERROR DELAY COUNTER" ;/ To find this routine in the asm file.
        EndAsm
        If F_PrintErrorDelay = 1 Then
            Dec dPrintErrorDelay
            If dPrintErrorDelay = 0 Then
                F_PrintErrorDelay = 0
            EndIf 
        EndIf

    EndIf
Context Restore					    '/ Exit the interrupt routine		
    Asm
    messg "END OF INTERRUPT ROUTINE" ;/ To find this routine in the asm file.
    EndAsm
'===============================================================================
'/ Name          : PrintDisp(pmodifier,pValue) 
'/ Input         : Modifier/Integer
'/ Purpose       : Declare some macros for Maths. (To avoid errata errors)
'/ Return Value  : None
'/ Parameters    : 
'/ Notes         : Print all variables. There is not any control of the Variable type.
'/               : "messg" = To find this macro in the asm file.
'/ 
$define PrintDisp(pmodifier,pValue) '
    Asm                             '
    messg "Macro - PrintDisp()"     '
    EndAsm                          '
    RsOut pmodifier pValue,CR

'/------------------------------------------------------------------------------
'/ Name          : PrintDispInt(pmodifier,pValue) 
'/ Input         : Modifier/Variable
'/ Purpose       : Declare some macros for Maths. (To avoid errata errors)
'/ Return Value  : None
'/ Parameters    : 
'/ Notes         : Print integer variables only. There is a control of the Variable type.
'/               : "messg" = To find this macro in the asm file.
'/ 
$define PrintDispInt(pmodifier,pValue) '
    Asm                                '
    messg "Macro - PrintDispInt()"     '
    EndAsm                             '
    RsOut pmodifier pValue,CR          '
    _mPrintDispInt pValue

'/ This macro is used for testing variables only.
_mPrintDispInt Macro- pValue
    #if (Prm_Count != 1)
        #error "_mPrintDispInt - This macro uses 1 parameter!"
    #else
        #if (Prm_1 != Dword) && (Prm_1 != Word) && (Prm_1 != Byte)
            #error "_mPrintDispInt Macro - (Param1) Should be a Dword, Word or Byte Variable"
        #endif
    #endif
Endm

'/------------------------------------------------------------------------------
'/ Name          : PrintDispSign(pmodifier,pValue) 
'/ Input         : Modifier/Integer
'/ Purpose       : Declare some macros for Maths. (To avoid errata errors)
'/ Return Value  : None
'/ Parameters    : 
'/ Notes         : Print integer variables only. There is a control of the Variable type.
'/               : "messg" = To find this macro in the asm file.
'/ 
$define PrintDispSign(pmodifier,pValue) '
    Asm                                 '
    messg "Macro - PrintDispSign()"     '
    EndAsm                              '
    RsOut pmodifier pValue,CR           '
    _mPrintDispSign pValue

'/ This macro is used for testing variables only.
_mPrintDispSign Macro- pValue
    #if (Prm_Count != 1)
        #error "_mPrintDispSign - This Macro uses 1 parameter!"
    #else
        #if (Prm_1 != SDword) && (Prm_1 != SWord) && (Prm_1 != SByte)
            #error "_mPrintDispSign Macro - (Param1) Should be a Signed SDword,  SWord or SByte Variable"
        #endif
    #endif
Endm

'/------------------------------------------------------------------------------
'/ Name          : PrintDispFlt(pmodifier,pValue) 
'/ Input         : Modifier/Float
'/ Purpose       : Declare some macros for Maths. (To avoid errata errors)
'/ Return Value  : None
'/ Parameters    : 
'/ Notes         : Print Float variables only. There is a control of the Variable type.
'/               : "messg" = To find this macro in the asm file.
'/ 
$define PrintDispFlt(pmodifier,pValue) '
    Asm                                '
    messg "Macro - PrintDispFlt()"     '
    EndAsm                             '
    RsOut pmodifier pValue,CR          '
    _mPrintDispFloat pValue

'/ This macro is used for testing variables only.
_mPrintDispFloat Macro- pValue
    #if (Prm_Count != 1)
        #error "_mPrintDispFloat - This Macro uses 1 parameter!"
    #else
        #if(Prm_1 != Float)
            #error "_mPrintDispFloat Macro - (Param1) Should be a Float Variable"
        #endif
    #endif
Endm
'===============================================================================
'===============================================================================		
'/ Name          : RsOut (new command)
'/ Input         : See PDS manual
'/ Purpose       : Disable the RsOut command To be changed For a Code to drive the 8 LEDs displays.
'/ Return Value  : See PDS manual
'/ Parameters    : Modifiers (See PDS manual)
'/ Notes         : Disable the RsOut command To be changed For a Code
'/               : Build up the seperate digits of input variable.
'/               : "messg" = To find this macro in the asm file.
'/               : The array dimension must to be [Number of Digits + 1] to allow the shift of the bytes. 
'/------------------------------------------------------------------------------
    #Disable RsOut
'/==============================================================================
'/ New RsOut Routine.
    Asm
    messg "RSOUT ROUTINE" ;/ To find this routine in the asm file.
    EndAsm
    Asm
    RsOut
    EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif
    If F_PrintErrorDelay = 1 Then
        Return                      '/ If error return to the RsOut command.
    EndIf
    INDF1 = WREG                    '/ Put the value into the array.
    dRsOutByteIn = WREG             '/ Save it.
    Inc dBytesReceived              '/ Increments Number of bytes received.

    $if TableData = 1
    EECON1 = 0                      '/ Eeprom to read. (only once is sufficient)
    $endif

    '/ Version 1.1. With a new data table distribution, more than 20 lines of code 
    '/ can be saved.
    '/ Checking the higher values first.
    Select dRsOutByteIn
        Case > 90   '/ Looking for no allowed characters values.
            '/ NO increment the pointer. The value is not valid. Goto Error only.
            GoTo SetAnErrorDisp
 
        Case > 47   '/ Looking for decimals variables, Hex variables an High Case ASCII characters.
            $if TableData = 0
                '/ Data from the ROM memory.
                WREG = dRsOutByteIn - 48            '/ Convert ASCIIdec to BCD.
                INDF1 = CRead8 SegmentsData[WREG]   '/ Data for segments.
            $else
                '/ Data from the eeprom memory.
                EEADR = dRsOutByteIn - 48   '/ Convert ASCIIdec to BCD.
                EECON1bits_RD = 1           '/ Set the eeprom to read.
                INDF1 = EEDATA              '/ Data for segments.
            $endif 
            Dec FSR1                        '/ Next array address to write.

        Case 46    '/ Looking for Dot ("."). (Floating DP)
            Inc FSR1                '/ Looking for the last valid digit received.
            INDF1.7 = 0             '/ Set the DP as for common Anode Display in the last digit received. 
            Dec FSR1                '/ Restore the FSR1 pointer for the next byte to receive.
            Dec dBytesReceived      '/ It is not a new value.

        Case 45    '/ Looking for Minus ("-").
            INDF1 = MinusCharacter  '/ Load directly the display value. Less lines of code.
            Dec FSR1                '/ Next array address to write.

        Case 13     '/ Test final bytes; looking For Carriage Return.
            INDF1 = 255                     '/ Clear the CR characters and blanck the position.   
            Dec dBytesReceived              '/ It is not a valid byte.
            GoTo CheckErrorBytesReceived    '/ All bytes received already.
    EndSelect
    Return                          '/ Come back for more bytes; The RsOut commant is in use.
'/------------------------------------------------------------------------------
CheckErrorBytesReceived:
'/ Check the number of bytes received. If YES then print "Error" in the display.
    If dBytesReceived > LenghtDISP_STRING - 1 Then
SetAnErrorDisp:
        DisPlayInterruptEnable = 0      '/ Avoid blinking.
        $if NumberDigits = 8
        PRINT_ONES = 175                '/ Character r
        PRINT_TENS = 163                '/ Character o
        PRINT_HUNDREDS = 175            '/ Character r
        PRINT_THOUSANDS = 175           '/ Character r
        PRINT_TENTHOUSANDS = 134        '/ Character E
        PRINT_HUNDREDTHOUSANDS = 255 
        PRINT_THOUSANDTHOUSANDS = 255
        PRINT_TENTHDTHOUSANDS = 255
        $endif
        $if NumberDigits = 7
        PRINT_ONES = 175                '/ Character r
        PRINT_TENS = 163                '/ Character o
        PRINT_HUNDREDS = 175            '/ Character r
        PRINT_THOUSANDS = 175           '/ Character r
        PRINT_TENTHOUSANDS = 134        '/ Character E
        PRINT_HUNDREDTHOUSANDS = 255 
        PRINT_THOUSANDTHOUSANDS = 255
        $endif
        $if NumberDigits = 6
        PRINT_ONES = 175               '/ Character r
        PRINT_TENS = 163               '/ Character o
        PRINT_HUNDREDS = 175           '/ Character r
        PRINT_THOUSANDS = 175          '/ Character r
        PRINT_TENTHOUSANDS = 134       '/ Character E
        PRINT_HUNDREDTHOUSANDS = 255 
        $endif
        $if NumberDigits = 5
        PRINT_ONES = 175               '/ Character r
        PRINT_TENS = 163               '/ Character o
        PRINT_HUNDREDS = 175           '/ Character r
        PRINT_THOUSANDS = 175          '/ Character r
        PRINT_TENTHOUSANDS = 134       '/ Character E
        $endif
        $if NumberDigits = 4
        PRINT_ONES = 175               '/ Character r
        PRINT_TENS = 175               '/ Character r
        PRINT_HUNDREDS = 134           '/ Character E
        PRINT_THOUSANDS = 255  
        $endif 
        $if NumberDigits = 3
        PRINT_ONES = 175               '/ Character r
        PRINT_TENS = 175               '/ Character r
        PRINT_HUNDREDS = 134           '/ Character E
        $endif 
        DisPlayInterruptEnable = 1  
        dPrintErrorDelay = VErrorDelayMS
        F_PrintErrorDelay = 1               '/ Flag for interrupt routine.
        GoTo RestoreArrayParameters_Sub
    EndIf
'/------------------------------------------------------------------------------
MoveDatasDispArray:
'/ Prepare the array & move bytes from high position to lower position. (LSB first)
    Dec dBytesReceived               '/ prepare FSR1
    FSR1 = dHighPointerFSR1          '/ Restore the FSR1 to higher byte of the array.
    FSR1 = FSR1 - dBytesReceived     '/ Point the FSR1 at the last byte received.
    FSR0 = VarPtr (DISP_STRING)      '/ Point the FSR0 to lower byte of the STRING.
    Inc dBytesReceived               '/ Restore the correct value of dBytesReceived.
    Repeat 
        #ifdef watchdog_req
        Clrwdt
        #endif
        INDF0 = INDF1                '/ Copy all bytes
        INDF1 = 255                  '/ Blanking old bytes
        Inc FSR0
        Inc FSR1
        Dec dBytesReceived
    Until dBytesReceived = 0         '/ Shift all the digits already.
'/------------------------------------------------------------------------------
CopyArrayToDisplay_Sub:
    ' Make a copy to print the values for the interrupt routine.
    DisPlayInterruptEnable = 0          '/ Avoid blinking.
    $if NumberDigits = 8
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    PRINT_TENTHOUSANDS = TENTHOUSANDS 
    PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
    PRINT_THOUSANDTHOUSANDS = THOUSANDTHOUSANDS
    PRINT_TENTHDTHOUSANDS = TENTHDTHOUSANDS
    $endif
    $if NumberDigits = 7
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    PRINT_TENTHOUSANDS = TENTHOUSANDS 
    PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
    PRINT_THOUSANDTHOUSANDS = THOUSANDTHOUSANDS
    $endif
    $if NumberDigits = 6
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    PRINT_TENTHOUSANDS = TENTHOUSANDS 
    PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
    $endif            
    $if NumberDigits = 5
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    PRINT_TENTHOUSANDS = TENTHOUSANDS 
    $endif              
    $if NumberDigits = 4
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    $endif 
    $if NumberDigits = 3
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    $endif
    DisPlayInterruptEnable = 1 
'/------------------------------------------------------------------------------
'/ Restore all parameters before to use the RsOut command again.
RestoreArrayParameters_Sub:
'/ Load all the array with blank characters.
'/ Clear DISP_STRING, blanking all the digits.
    FSR0 = VarPtr (DISP_STRING)
    PRODH = LenghtDISP_STRING
    Repeat
        #ifdef watchdog_req
        Clrwdt
        #endif
        POSTINC0 = 255
        Dec PRODH
    Until PRODH = 0
    INDF0 = 0                           '/ Set Array as String again.

'/ Configure the PORTs again in case of...
'    DisPlayInterruptEnable = 0
	Output SEGMENTS             '/ Put the display PORT as Output.
    '/ Every individual pin is set as output.
    Output AddressDA
    Output AddressDB
    Output AddressDC
    Output LE2
    '/ Every individual pin is set as input.
    $ifdef _Keypad
    Input KA
    Input KB    
    Input KC
    Input KD
    $endif
'    DisPlayInterruptEnable = 1

'/ Clear some variables.
    dBytesReceived = 0

'/ Configure the array pointers To start a new RsOut command.
    FSR0 = VarPtr (DISP_STRING)              '/ FSR0 At the beginning of the array.
    FSR1 = FSR0 + (LenghtDISP_STRING - 1)    '/ FSR1 at the end of the array to start a new RsOut command.
    dHighPointerFSR1 = FSR1                 '/ Make a copy of FRS1.
    Return                                  '/ End of routine RsOut.
'===============================================================================
'===============================================================================

'===============================================================================
_MultiplexDigitsExit:
'===============================================================================
    DisplayInit()       '/ Initialize the Displays.
'===============================================================================
$endif  ' _MultiplexDigits
'===============================================================================





